/*
*****************************************************************************************************************************************************
Projet Centrale domotique DomoMaison 2017 pour carte Arduino Mega 2560 et carte Ethernet (W5100)
Version 1.70 du 06/09/2017
Copyright 2017 PRIN Jean-Luc
Ce programme fonctionne avec le protocole Websockets (HTML5).
*****************************************************************************************************************************************************
*/
/*
*****************************************************************************************************************************************************
Modules contenant les fonctions nécessaires à la gestion des fils pilote et du chauffage
*****************************************************************************************************************************************************
*/

#ifdef CHAUFCONF
// ------------------------- Début de la fonction envoi_mode_radia (envoi du mode de fonctionnement des radiateurs) ---------------------------------
void envoi_mode_radia()
{
    // Elaboration du message
    EvtServer[0] = STX;
    strncpy(&EvtServer[1], NumCard, 4);
    EvtServer[5] = Separator;
    strncpy(&EvtServer[6], "RADIA",5);
    EvtServer[11] = Separator;
    strncpy(&EvtServer[12], "01",2);                    // 01 pour envoi mode de fonctionnement
    EvtServer[14] = Separator;
        for (int i=0; i <8; i++) {
			snprintf(&EvtServer[15 + i],2,"%1d",ModeCours_Radia[i]);
        }
    EvtServer[23]= ETX;
    EvtServer[24]= '\0';
        #ifdef DEBUG_CHAUF
            Serial.print(F("chaine emise = "));
            Serial.println(EvtServer);
        #endif
        if (wsServer.isConnected()) {
            wsServer.send(EvtServer,24);                //Envoi
        }
}
// ------------------------- Fin de la fonction envoi_mode_radia (envoi du mode de fonctionnement des radiateurs) -----------------------------------

// -------------------------- Début de la fonction envoi_program_radia (envoi de la programmation des radiateurs) -----------------------------------
void envoi_program_radia()
 {

    byte i, j;
    int Temp;

    // Elaboration du message sous la frme (ME01;RADIA;02;01;0;32/32/2032;02130400;12;40)
        for (i=0; i<8; i++) {
            EvtServer[0] = STX;
            strncpy(&EvtServer[1], NumCard, 4);
            EvtServer[5] = Separator;
            strncpy(&EvtServer[6], "RADIA;02",8);               // 02 pour signaler envoi programmation
            EvtServer[14] = Separator;
			snprintf(&EvtServer[15],3,"%02d",i + 1);
            EvtServer[17] = Separator;
            Temp = EEPROM.read(1400 + (i * 22));
			snprintf(&EvtServer[18],2,"%1d",Temp);
            EvtServer[19] = Separator;
            Temp = EEPROM.read(1401 + (i * 22));
			snprintf(&EvtServer[20],3,"%02d",Temp);             // jour de la programmation (jour ou 32 pour semaine, 33 pour week-end
            EvtServer[22] = '/';
            Temp = EEPROM.read(1402 + (i * 22));
			snprintf(&EvtServer[23],3,"%02d",Temp);             // mois de la programmation (mois ou 32 pour semaine, 33 pour week-end
            EvtServer[25] = '/';
            Temp = (EEPROM.read(1403 + (i * 22))) + 2000;       // passage de l'année sur 4 chiffres
			snprintf(&EvtServer[26],5,"%4d",Temp);              // annee de la programmation (annee ou 32 pour semaine, 33 pour week-end
            EvtServer[30] = Separator;
                for (j=0; j<8; j++) {
                    Temp = EEPROM.read(1404 + (i * 22) + j);
                    snprintf(&EvtServer[31 + j],2,"%1d",Temp);  // mode de fonctionnement des radiateurs (0 à 4)
                }
            EvtServer[39] = Separator;
            Temp = EEPROM.read(1420 + (i * 22));
			snprintf(&EvtServer[40],3,"%02d",Temp);             // heure d'application (99 si pas de sélection)
            EvtServer[42] = Separator;
            Temp = EEPROM.read(1421 + (i * 22));
			snprintf(&EvtServer[43],3,"%02d",Temp);              // minute d'application (99 si pas de sélection)
            EvtServer[45] = ETX;
            EvtServer[46] = '\0';
            #ifdef DEBUG_CHAUF
                Serial.print(F("chaine emise = "));
                Serial.println(EvtServer);
            #endif
            wsServer.send(EvtServer,46);
        }
}
// -------------------------- Fin de la fonction envoi_program_radia (envoi de la programmation des radiateurs) -------------------------------------

// -------------------- Début de la fonction commandeEjp_Rad (commande des 8 zones des radiateurs sur Arrêt lors du tarif EJP) ----------------------
void commandeEjp_Rad()
{
    byte i;

        // envoi du 2220V simple alternance positive sur tous les fils pilote de tous les radiateurs (Arrêt des radiateurs)
        for (i=0; i < 4; i++){
            expander1.digitalWrite(i*2, HIGH);
            expander1.digitalWrite((i*2)+1, LOW);
        }
        for (i=0; i < 4; i++){
            expander2.digitalWrite(i*2, HIGH);
            expander2.digitalWrite((i*2)+1, LOW);
        }
        // incation de fonctionnement des radiateurs sur Arrêt
        for (i=0; i <8; i++){
            ModeCours_Radia[i] = 4;
        }
    // ajoute un jour EJP au compteur de jour mis en méméoire EEPROM
    i = EEPROM.read(112);
    i++;
    EEPROM.write(112, i);
    BascuEjp = Ejp;             // mise en mémoire de l'état Ejp
}
// -------------------- Fin de la fonction commandeEjp_Rad (commande des 8 zones des radiateurs sur Arrêt lors du tarif EJP) ------------------------

// ------------------ Début de la fonction commande_immediat_radia (commande des radiateurs zone par zone) ------------------------------------------
void commande_immediat_radia()
{
    byte i;

    // On recharge de mode de fonctionnement immédiat des radiateurs
        for (i=0; i<8; i++) {
            ModeCours_Radia[i] = EEPROM.read(80 + i);
        }
    // Zones 1 à 4
        for (i=0; i<4; i++) {
			switch (ModeCours_Radia[i]) {
				case 0:                                         // Non connecté
                    expander1.digitalWrite(i*2, HIGH);
                    expander1.digitalWrite((i*2)+1, HIGH);
					break;
				case 1:                                         // Confort
                    expander1.digitalWrite(i*2, HIGH);
                    expander1.digitalWrite((i*2)+1, HIGH);
					break;
				case 2:                                         // Economique
                    expander1.digitalWrite(i*2, LOW);
                    expander1.digitalWrite((i*2)+1, LOW);
					break;
				case 3:                                         // Hors Gel
                    expander1.digitalWrite(i*2, LOW);
                    expander1.digitalWrite((i*2)+1, HIGH);
					break;
				case 4:                                         // Arrêt
                    expander1.digitalWrite(i*2, HIGH);
                    expander1.digitalWrite((i*2)+1, LOW);
					break;
				default:
					break;
			}
        }

    // Zones 5 à 8
        for (i=0; i<4; i++) {
			switch (ModeCours_Radia[i + 4]) {
				case 0:                                         // Non connecté
                    expander2.digitalWrite(i*2, HIGH);
                    expander2.digitalWrite((i*2)+1, HIGH);
					break;
				case 1:
                    expander2.digitalWrite(i*2, HIGH);          // Confort
                    expander2.digitalWrite((i*2)+1, HIGH);
					break;
				case 2:                                         // Economique
                    expander2.digitalWrite(i*2, LOW);
                    expander2.digitalWrite((i*2)+1, LOW);
					break;
				case 3:                                         // Hors Gel
                    expander2.digitalWrite(i*2, LOW);
                    expander2.digitalWrite((i*2)+1, HIGH);
					break;
				case 4:                                         // Arrêt
                    expander2.digitalWrite(i*2, HIGH);
                    expander2.digitalWrite((i*2)+1, LOW);
					break;
				default:
					break;
			}
        }

    BascuEjp = 0;                                               // pour retour à l'EJP si présent
}
// ------------------ Fin de la fonction commande_immediat_radia (commande des radiateurs zone par zone) --------------------------------------------

// ----------- Début de la fonction enregImmediat_Rad (enregistrement du fonctionnement immédiat ou par défaut des radiateurs) ----------------------
void enregImmediat_Rad(char* Chaine)
{

    byte Tp;

        for (byte i=0; i<8; i++) {
            Tp = atoi(&Chaine[15 + (i*2)]);
            EEPROM.write(80 + i,Tp);
            ModeCours_Radia[i] = Tp;
        }
    creMessage("01","EnvoiOK", 7);

    commande_immediat_radia();               // commande les radiateurs en mode de fonctionnement immédiat
}
// ----------- Fin de la fonction enregImmediat_Rad (enregistrement du fonctionnement immédiat ou par défaut des radiateurs) ------------------------

// ------------- Début de la fonction enregProg_Rad (enregistrement de la programmation du fonctionnement des radiateurs) ---------------------------
void enregProg_Rad(char* Chaine)
{
    // Message reçu sous la forme :(ME01;FORA;02;01;0;12/10/2017;00113041;00;00)
    // mais tronqué dans cette fonction à : 01;0;12/10/2017;00113041;00;00)

    char Buff[] = "xx";
    byte Chiffre;
    byte NumProg;
    int Adresse;

    Buff[0] = Chaine[15];                                   // Extraction d'un numéro de programme
    Buff[1] = Chaine[16];
    Buff[2] = '\0';
    NumProg = atoi(Buff);
    Adresse = 1400 + ((NumProg - 1) * 22);
    Buff[0] = Chaine[18];                                   // Extraction de l'activation (0 ou 1)
    Buff[1] = '\0';
    Chiffre = atoi(Buff);
    EEPROM.write(Adresse, Chiffre);
    Buff[0] = Chaine[20];                                   // Extraction du jour de programmation)
    Buff[1] = Chaine[21];
    Buff[2] = '\0';
    Chiffre = atoi(Buff);
    EEPROM.write(Adresse + 1, Chiffre);
    Buff[0] = Chaine[23];                                   // Extraction du mois de programmation
    Buff[1] = Chaine[24];
    Buff[2] = '\0';
    Chiffre = atoi(Buff);
    EEPROM.write(Adresse + 2, Chiffre);
    Buff[0] = Chaine[26];                                   // Extraction de l'année de programmation (sur 2 chiffres)
    Buff[1] = Chaine[27];
    Buff[2] = '\0';
    Chiffre = atoi(Buff);
    EEPROM.write(Adresse + 3, Chiffre);
        for (byte i=0; i <8; i++){
            Buff[0] = Chaine[31 + i];                       // Extraction du mode de fonctionnement des radiateurs (0 ou 4)
            Buff[1] = '\0';
            Chiffre = atoi(Buff);
            EEPROM.write(Adresse + 4 + i, Chiffre);
        }
    Buff[0] = Chaine[40];                                   // Extraction de l'heure d'application du programme
    Buff[1] = Chaine[41];
    Buff[2] = '\0';
    Chiffre = atoi(Buff);
    EEPROM.write(Adresse + 20, Chiffre);
    Buff[0] = Chaine[43];                                   // Extraction des minutes d'application du programme
    Buff[1] = Chaine[44];
    Buff[2] = '\0';
    Chiffre = atoi(Buff);
    EEPROM.write(Adresse + 21, Chiffre);

        if (NumProg == 8) {
            creMessage("01", "EnvoiOK", 7);
            charger_prog_radia();                           // chargement de la programmation des radiateurs pour la journée
        }
}
// ------------- Fin de la fonction enregProg_Rad (enregistrement de la programmation du fonctionnement des radiateurs) -----------------------------

// ---------------- Début de la onction charger_prog_radia (chargement de la programmation des radiateurs) ------------------------------------------
void charger_prog_radia() {

    byte Temp;
    boolean Act;
    byte JourProg;
    byte MoisProg;
    byte AnneeProg;
    int Hretemp;
    int Mintemp;
    int Inc = 0;
    boolean Jok;

    NbProgV = 0;

        for (byte i=0; i<8; i++) {
            Act = EEPROM.read(1400 + (i*22));                                               // activation du programme
                if (Act == 1) {                                                             // programme activé
                    JourProg = EEPROM.read(1401 + (i*22));                                  // jour de la programmation (1 à 31)
                    MoisProg = EEPROM.read(1402 + (i*22));                                  // mois du programme
                    AnneeProg = EEPROM.read(1403 + (i*22));                                 // annee du programme (sans le millésime)
                    // recherche d'abord si la journée courante est concernée par une programmation des volets
                    Jok = false;
                        if ((JourCourant == 6 || JourCourant == 7) && JourProg == 33) {             // 33 = wek-end
                            Jok = true;
                        }
                        else if ((JourCourant > 0 && JourCourant < 6) && JourProg == 32) {          // 32 = semaine
                            Jok = true;
                        }
                        else if ((JourProg == DateJour) && (MoisProg == Mois) && ((AnneeProg + 2000) == Annee)) {
                            Jok = true;
                        }
                        if (Jok == true) {
                            NumProgRad[Inc] = i + 1;
                            Temp = EEPROM.read(1420 + (i * 22));
                                if (Temp != 99) {
                                    Hretemp = Temp;
                                    Temp = EEPROM.read(1421 + (i * 22));
                                    Mintemp = Temp;
                                    HreProgRad[Inc] = (Hretemp * 60) + Mintemp;
                                }
                                else {
                                    HreProgRad[Inc] = 1500;
                                }
                            #ifdef DEBUG_CHAUF
                                Serial.print(F("Prog. Rad = "));
                                Serial.print(NumProgRad[Inc]);
                                Serial.print(F(";"));
                                Serial.print(HreProgRad[Inc]);
                            #endif
                        Inc = Inc + 1;
                    }
                }
        }
    NbreProgRad = Inc;
}
// ---------------- Fin de la onction charger_prog_radia (chargement de la programmation des radiateurs) --------------------------------------------

// ----------------- Début de la fonction applique_prog_radia (application de la programmation des radiateurs) --------------------------------------
void applique_prog_radia() {

    byte i, j;
    byte Prog;
    byte ModeFonc;
    int Adresse;

        if (NbreProgRad > 0) {                                              // si au moins un programme pour les radiateurs
            for (i=0; i < NbreProgRad; i++){
                if (HreProgRad[i] <= TempsMn) {
                    Prog = NumProgRad[i];
                    Adresse = 1404 + ((Prog - 1) * 22);                     // adresse dans l'EEPROM
                    #ifdef DEBUG_CHAUF
                        Serial.println(F("Programmation radiateurs"));
                    #endif
                    for (j=0; j<4; j++) {                                   // lecture des 8 zones
                        ModeFonc = EEPROM.read(Adresse + j);
                        ModeCours_Radia[j] = ModeFonc;
                        // Zones 1 à 4
                            switch (ModeFonc) {
                                case 0:                                     // non connecté
                                    expander1.digitalWrite(j*2, HIGH);
                                    expander1.digitalWrite((j*2)+1, HIGH);
                                    break;
                                case 1:                                     // Confort
                                    expander1.digitalWrite(j*2, HIGH);
                                    expander1.digitalWrite((j*2)+1, HIGH);
                                    break;
                                case 2:                                     // Economique
                                    expander1.digitalWrite(j*2, LOW);
                                    expander1.digitalWrite((j*2)+1, LOW);
                                    break;
                                case 3:                                     // Hors Gel
                                    expander1.digitalWrite(j*2, LOW);
                                    expander1.digitalWrite((j*2)+1, HIGH);
                                    break;
                                case 4:                                     // Arrêt
                                    expander1.digitalWrite(j*2, HIGH);
                                    expander1.digitalWrite((j*2)+1, LOW);
                                    break;
                                default:
                                    break;
                            }
                    }
                    for (j=0; j<4; j++) {
                        ModeFonc = EEPROM.read(Adresse + j + 4);
                        ModeCours_Radia[j + 4] = ModeFonc;
                        // Zones 5 à 8
                            switch (ModeFonc) {
                                case 0:                                     // non connecté
                                    expander2.digitalWrite(j*2, HIGH);
                                    expander2.digitalWrite((j*2)+1, HIGH);
                                    break;
                                case 1:                                     // Confort
                                    expander2.digitalWrite(j*2, HIGH);
                                    expander2.digitalWrite((j*2)+1, HIGH);
                                    break;
                                case 2:                                     // Economique
                                    expander2.digitalWrite(j*2, LOW);
                                    expander2.digitalWrite((j*2)+1, LOW);
                                    break;
                                case 3:                                     // Hors Gel
                                    expander2.digitalWrite(j*2, LOW);
                                    expander2.digitalWrite((j*2)+1, HIGH);
                                    break;
                                case 4:                                     // Arrêt
                                    expander2.digitalWrite(j*2, HIGH);
                                    expander2.digitalWrite((j*2)+1, LOW);
                                    break;
                                default:
                                    break;
                            }
                    }
                    HreProgRad[i] = 1500;                                   // permet de considérer l'action effectuée
                }
            }
        }
}
// ----------------- Fin de la fonction applique_prog_radia (application de la programmation des radiateurs) ----------------------------------------
#endif

